"""
@Author: shuangchen
@Time：2024/3/12
@File: selenium.py
@Description: 封装selenium的一些操作，自动获取cookies，隐藏特征
"""

import os
import requests
from selenium import webdriver
from selenium.webdriver.chrome.options import Options


class PageNotFound(Exception):
    pass


class SeleniumSpider:
    def __init__(self, url: str, cookies: str = '') -> None:
        """
        :param url: 网页链接
        :param cookies: 全局的cookies值
        """
        self.base_url = url
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
            'Cookie': cookies
        }

    def _get_client_cookies(self) -> None:
        """
        获取客户端的cookies
        :return: None
        """
        os.system(r'start chrome --remote-debugging-port=9527 --user-data-dir="D:\ChromeAutomationProfile"')
        options = Options()
        options.add_argument('--headless')
        options.add_experimental_option("debuggerAddress", "127.0.0.1:9527")
        chrome = webdriver.Chrome(options=options)
        chrome.get(self.base_url)
        chrome.refresh()
        cookies = ''
        for c in chrome.get_cookies():
            cookies += f'{c["name"]}={c["value"]};'
        self.headers['Cookie'] = cookies

    def get_html(self, url=None) -> str:
        """
        获取网页源码
        :param url: 网页链接
        :return: 网页源码
        """
        if url is None:
            url = self.base_url
        try:
            r = requests.get(url, headers=self.headers)
        except requests.exceptions.ConnectTimeout:
            raise PageNotFound('网页不存在: ' + url)
        else:
            if r.status_code == 412 or r.status_code == 202:
                self._get_client_cookies()
                return self.get_html(url)
            elif r.status_code == 404 or r.status_code == 500:
                raise PageNotFound('网页不存在: ' + url)
            r.encoding = 'utf-8'
            return r.text

    def get_content(self, url=None) -> bytes:
        """
        获取文件内容
        :param url: 文件链接
        :return: 文件内容
        """
        if url is None:
            url = self.base_url
        try:
            r = requests.get(url, headers=self.headers)
        except requests.exceptions.ConnectTimeout:
            raise PageNotFound('文件不存在: ' + url)
        else:
            if r.status_code == 412 or r.status_code == 202:
                self._get_client_cookies()
                return self.get_content(url)
            elif r.status_code == 404:
                raise PageNotFound('文件不存在: ' + url)
            return r.content



"""
@Author: shuangchen
@Time：2024/3/12
@File: doc_segment_by_rule.py
@Description: 通过规则分割文档
"""

import re

PATTERN_NUMBER_ERROR_MSG = '分割的深度应当与用以匹配标题的规则数量相同'
SEGMENT_DEPTH_ERROR_MSG = '分割的深度应当在1-3之间'
FILE_TYPE_ERROR_MSG = '只支持.txt格式'


def merge_sentences(sentences, title_pattern: str = None) -> list[str]:
    """
    合并句子的不正确的换行，以句尾标点为依据
    :param sentences: 合并前句子列表
    :param title_pattern: 需要忽略合并的句子规则（一般为标题）
    :return: 合并前句子列表
    """
    merged = []
    temp = ''
    for sentence in sentences:
        if title_pattern is None and re.match(title_pattern, sentence):
            if temp:
                merged.append(temp)
                temp = ''
            merged.append(sentence)
        else:
            if not re.search(r'[.!?。，？；！:：）)]$', sentence):  # $确定表达符号处于句子的最后
                temp += sentence
            else:
                if temp:
                    merged.append(temp + sentence)
                    temp = ''
                else:
                    merged.append(sentence)
    if temp:
        merged.append(temp)
    return merged


class DocSegmentByRule:
    def __init__(self, depth: int, patterns: list[str], merge_sentences: bool = True) -> None:
        """
        使用规则匹配文章标题，按标题进行分割
        :param depth: 分割深度，支持的深度为1-3
        :param patterns: 标题匹配规则，按一级标题、二级标题...顺序排列
        :param merge_sentences: 是否进行句子不正确换行的合并
        """
        assert depth == len(patterns), PATTERN_NUMBER_ERROR_MSG
        assert 1 <= depth <= 3, SEGMENT_DEPTH_ERROR_MSG
        self.depth = depth
        self.patterns = patterns
        self.title_pattern = '|'.join([f'({p})' for p in patterns])  # 用于合并句子时忽略标题
        self.merge_sentences = merge_sentences

    def solve(self, files: str | list[str]) -> list[str] | dict[str, list[str]]:
        """
        对文件进行分割
        :param files: 文件或文件列表
        :return: 每份文件的段落列表，以字典文件名：列表格式返回
        """
        if isinstance(files, str):
            files = [files]

        result = {}

        for file in files:
            assert file.endswith('.txt'), FILE_TYPE_ERROR_MSG
            f = open(file, 'r', encoding='utf-8')
            lines = [line.strip() for line in f.readlines()]
            if self.merge_sentences:
                lines = merge_sentences(lines, self.title_pattern)
            f.close()

            paragraphs = []
            temp = ''

            if self.depth == 1:
                for line in lines:
                    if re.match(self.patterns[0], line):  # 一级标题
                        if temp:
                            paragraphs.append(temp.rstrip())
                            temp = ''
                    temp += line + '\n'
                if temp:
                    paragraphs.append(temp.rstrip())

            elif self.depth == 2:
                title1 = ''
                for line in lines:
                    if re.match(self.patterns[1], line):  # 二级标题
                        if temp:
                            paragraphs.append(title1 + '\n' + temp.rstrip())
                            temp = ''
                        temp += line + '\n'
                    elif re.match(self.patterns[0], line):  # 一级标题
                        if temp:
                            paragraphs.append(title1 + '\n' + temp.rstrip())
                            temp = ''
                        elif len(title1) != 0:
                            paragraphs.append(title1)
                        title1 = line
                    else:  # 正文
                        temp += line + '\n'
                if temp:
                    paragraphs.append(title1 + '\n' + temp.rstrip())
                elif len(title1) != 0:
                    paragraphs.append(title1)

            elif self.depth == 3:
                title1, title2 = '', ''
                for line in lines:
                    if re.match(self.patterns[2], line):  # 三级标题
                        if temp:
                            paragraphs.append(title1 + '\n' + title2 + '\n' + temp.rstrip())
                            temp = ''
                        temp += line + '\n'  # 这里直接写成temp = line + '\n'也是可以的（上面的temp = ''也可以不要了）
                    elif re.match(self.patterns[1], line):  # 二级标题
                        if temp:
                            paragraphs.append(title1 + '\n' + title2 + '\n' + temp.rstrip())
                            temp = ''
                        elif len(title2) != 0:
                            paragraphs.append(title1 + '\n' + title2)
                        title2 = line
                    elif re.match(self.patterns[0], line):  # 一级标题
                        if temp:
                            paragraphs.append(title1 + '\n' + title2 + '\n' + temp.rstrip())
                            temp = ''
                        elif len(title2) != 0:
                            paragraphs.append(title1 + '\n' + title2)
                        elif len(title1) != 0:
                            paragraphs.append(title1)
                        title1 = line
                        title2 = ''
                    else: # 正文
                        temp += line + '\n'
                if temp:
                    paragraphs.append(title1 + '\n' + title2 + '\n' + temp.rstrip())
                elif len(title2) != 0:
                    paragraphs.append(title1 + '\n' + title2)
                elif len(title1) != 0:
                    paragraphs.append(title1)

            result[file] = paragraphs

        if len(result) == 1:
            return result[files[0]]
        return result


if __name__ == '__main__':
    pass



"""
@Author: shuangchen
@Time：2024/3/12
@File: conv.py
@Description: 各类转换工具函数
"""

NUMBER_TOO_LONG_ERR_MSG = '数字最大99999'


def numeral2chinese(number: int) -> str:
    """
    阿拉伯数字转中文表达
    :param number: 待转换的数字
    """
    assert number <= 99999, NUMBER_TOO_LONG_ERR_MSG
    num_dict = {1: '一', 2: '二', 3: '三', 4: '四', 5: '五', 6: '六', 7: '七', 8: '八', 9: '九', 0: '零'}
    digit_dict = {1: '十', 2: '百', 3: '千', 4: '万'}

    def max_digit(number, count):
        num = number // 10  # 整除
        if num != 0:
            return max_digit(num, count + 1)
        else:
            digit_num = number % 10  # digit_num是最高位上的数字
            return count, digit_num  # count记录最高位

    max_digit, digit_num = max_digit(number, 0)

    temp = number
    num_list = []  # 储存各位数字（最高位的数字也可以通过num_list[-1]得到
    while temp > 0:
        position = temp % 10
        temp //= 10  # 整除是//
        num_list.append(position)

    chinese = ""
    if max_digit == 0:  # 个位数
        chinese = num_dict[number]
    elif max_digit == 1:  # 十位数
        if digit_num == 1:  # 若十位上是1，则称为“十几”，而一般不称为“一十几”（与超过2位的数分开讨论的原因）
            chinese = "十" + num_dict[num_list[0]]
        else:
            chinese = num_dict[num_list[-1]] + "十" + num_dict[num_list[0]]
    elif max_digit > 1:  # 超过2位的数
        while max_digit > 0:
            if num_list[-1] != 0:  # 若当前位上数字不为0，则加上位称
                chinese += num_dict[num_list[-1]] + digit_dict[max_digit]
                max_digit -= 1
                num_list.pop(-1)
            else:  # 若当前位上数字为0，则不加上位称
                chinese += num_dict[num_list[-1]]
                max_digit -= 1
                num_list.pop(-1)
        chinese += num_dict[num_list[-1]]

    while chinese.endswith("零") and len(chinese) > 1:  # 个位数如果为0，不读出
        chinese = chinese[:-1]
    if chinese.count("零") > 1:  # 中文数字中最多只有1个零
        count_0 = chinese.count("零")
        chinese = chinese.replace("零", "", count_0 - 1)
    return chinese



import setuptools

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='shuangchentools',
    version='0.0.1',
    author='shuangchen',
    author_email='wangtao.cpu@gmail.com',
    description='encapsulated some commonly used tools',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/wangtao2001/sctools',
    packages=setuptools.find_packages(),
    install_requires=[
        'selenium>=4.15.2',
        'requests>=2.31.0'
    ]
)
